#!/bin/csh -f
# MacMetaMod.csh MRH Nov 06
# Modify meta data in image file EXIF data
# Runs on command line or via Platypus droplet
# Dependent on exifmod.csh and exiftool package
# exif_progress_filter and cocoadialog for Platypus
# This file is duplicated in Platypus Resource dir
# for completeness (main code is encrypted).
# Note that Platypus seems to execute .cshrc 
# regardless of -f flag above.
# =================================================
# =================================================

# ==========================
# Set constants and defaults
# ==========================

set echo

onintr exit

set prog_name = "Macmetamod.app";
set version = "v3.0";
set credit = "Mark Harris Mar 2020"

set meter = "1" # Enable progress meter

# App path is no longer first param, so we should put it back by hand so we don't have to change code later.
# But always an app

set pashua_flag = "1" # enable Pashua interface

# Set by hand to my desktop for debugging on OS 14

set appPath = $cwd/$prog_name
set os = `sw_vers -productVersion | cut -c 4-5`
if ( $os > 14 ) then
  if !(-f $cwd/Macmetamod.app/Contents/Info.plist)  cd ../../..
  set appPath = $cwd/$prog_name
  ##set appPath = "../../../$prog_name"
endif

echo " "
echo "Current working dir "
pwd
echo " "

## End 64 bit mods


set overwrite_string = "";
if (`uname -s` == 'Darwin') set overwrite_string = "-overwrite ";
set maxWordLength = 4090; # Shell limit

set platypus_flag = 0;
set prompt_string = "Enter keywords separated by commas : "
set append = " -append";
set single = "0"
set bubble_flag = 0;
set author = "";
set city = "";
set keywords = "";
set title = "";
set location = "";
set extra_msg = "";
set hard_key = ""
set hard_loc = ""
set hard_tag = ""
set hard_tit = ""
set saved_keywords = ""
set showimage = "1"
set multiprompt = ""
set individualprompt = ""
set firstArg = ""
set abort = ""

# Check if Platypus droplet, then if prog name encodes extras
# ===========================================================

set platypus_flag = 1;

set prog = `echo $prog_name | sed -e s/"\.[^\.]*"//`

if ($#argv > 0) then
set firstArg = "$argv[1]"

if ("$firstArg" == "multiprompt") then
  set individualprompt = "1"
  shift
endif

if (`echo $argv[1] | grep -i $prog | wc -l` || "$individualprompt" == "1") then # first arg contains prog name
    set platypus_flag = 1
    set platypusFirstArg = "$argv[1]"
    set prog_extras = `echo $argv[1] | sed s/".*$prog"// | sed -e s/'.app$'// -e s/'^[^-]*-'/-/`
    if ("$prog_extras" != "") then # Something appended to prog name
     if (`echo $prog_extras | grep -i "\-key" | wc -l`) then # -key found
	set prm = 'key'
	set hard_key = `echo $argv[1] | sed -e s/"^.*-$prm\_"// -e s/'[\.-].*'//`
	set keywords = "$hard_key"
	set hard_tag = "set"
     endif
     if (`echo $prog_extras | grep -i "\-loc" | wc -l`) then # -loc found
	set prm = 'loc'
	set hard_loc = `echo $argv[1] | sed -e s/"^.*-$prm\_"// -e s/'[\.-].*'//`
	set location = "$hard_loc"
	set hard_tag = "set"
     endif
     if (`echo $prog_extras | grep -i "\-tit" | wc -l`) then # -tit found
	set prm = 'tit'
	set hard_tit = `echo $argv[1] | sed -e s/"^.*-$prm\_"// -e s/'[\.-].*'//`
	set title = "$hard_tit"
	set hard_tag = "set"
     endif
    endif
endif
endif

# See if other programs are bundled
# =================================

if ("$platypus_flag" == "1") then
    set bundle = "0"

    if (-f "$appPath/Contents/Resources/Image-ExifTool-stripped/exiftool") set bundle = "$appPath";

	set exifmod = "$appPath/Contents/Resources/exifmod.csh -bundle $bundle";
	set exiftool = "$appPath/Contents/Resources/Image-ExifTool-stripped/exiftool";
	###set macmetamod = "$appPath/Contents/MacOS/Macmetamod"; # Needs second copy. Not this one.
	set macmetamod = "$appPath/Contents/Resources/macmetamod_64.csh"; # Needs second copy
	set progress_filter = "$appPath/Contents/Resources/exif_progress_filter.csh"
	set ccd = "$appPath/Contents/Resources/CocoaDialog.app/Contents/MacOS/CocoaDialog"
	set pashua = "$appPath/Contents/Resources/Pashua.app/Contents/MacOS/Pashua"
	set pashua_config_template = "$appPath/Contents/Resources/pashua_config.template"
	set pashua_config = "$appPath/Contents/Resources/pashua_config.txt"
	set pashua_output = "$appPath/Contents/Resources/pashua_output.txt"
	set pashua_welcome = "$appPath/Contents/Resources/pashua_welcome.txt"
	set pashua_toomany = "$appPath/Contents/Resources/pashua_too_many_files.txt"
	set pashua_badtype = "$appPath/Contents/Resources/pashua_badtype.txt"
	set filelist = "$appPath/Contents/Resources/filelist.txt"
	set pashua_saved_keywords = "$appPath/Contents/Resources/pashua_saved_keywords.txt"
	set icon = "$appPath/Contents/Resources/appIcon.icns"
	set prefs_file = "$appPath/Contents/Resources/mmm_preferences.txt"
	set setprefs = "$appPath/Contents/Resources/set_preferences.csh"
 
    ##shift # Platypus offset. Important in 32 bit, not wanted in 64 !

# If no files, show usage, offer prefs
# ====================================

# Welcome screen if no params

if ($#argv == 0) then
  $pashua "$pashua_welcome"
  goto exit
endif

# Get prefs from file if found
# ============================

if (-r "$prefs_file") then
    set author = `cat $prefs_file | grep "Author::" | sed -e s/"Author::"//`
    set city = `cat $prefs_file | grep "City::" | sed -e s/"City::"//`
endif

if ("$hard_tag" != "") goto doit # Skip prompting if something is hardcoded

# If only one file, show current keywords
# =======================================

set nonomatch

set curr_keys = ""
set curr_title = ""
set curr_author = ""
set curr_city = ""

if ($#argv == 1) then
  if (! -d "$argv[1]") then
    set single = "1";
    #echo `date`

    if ("$argv[1]:e" == "xmp") goto exit;

    set exifo = `$exiftool "$argv[1]" -keywords -title -creator |& grep -v thiswillneverbetrue`
    set exifo = `echo $exifo | sed -e s/"Creator City"/CreatorCity/ ` # Remove space to avoid match with Creator

    if (`echo $exifo | grep "Unknown image type"` != "") then
	          $pashua "$pashua_badfile"
        goto exit
    endif


    if (`echo $exifo | grep Keywords` != "") then
    #echo "ck1 $curr_keys " >> /tmp/mmm46
	set curr_keys = `echo $exifo | sed -e s/"Creator.*"// -e s/"CreatorCity.*"// -e s/"Title.*"// `
    #echo "ck2 $curr_keys " >> /tmp/mmm46
    endif

    if (`echo $exifo | grep Title` != "") set curr_title = `echo $exifo | sed -e s/"Creator.*"// -e s/"^.*Title"/\1/`

    if (`echo $exifo | grep "Creator "` != "") set curr_author = `echo $exifo | sed -e s/"^.*Creator "/\1/`
    if (`echo $exifo | grep CreatorCity` != "") set curr_city = `echo $exifo | sed -e s/"^.*CreatorCity"// | sed -e s/"Creator.*"//`

    set curr_keys = "`echo $curr_keys | sed s/"^[^:]*:"//`"
    set curr_title = "`echo $curr_title | sed s/"^[^:]*:"//`"
    set curr_author = "`echo $curr_author | sed s/"^[^:]*:"//`"
    set curr_city = "`echo $curr_city | sed s/"^[^:]*:"//`"
    set curr_city_string = ""
    if ("$curr_city" != "") set curr_city_string = ", $curr_city"

    #"$ccd" bubble --title "MacMetaMod" --text "$curr_author : $author" -icon-file "$icon" --timeout & 

	if ($platypus_flag != 1) then
     echo `date`
     echo "Current keys = $curr_keys"
     echo "Current title = $curr_title"
     echo "Current author = $curr_author"
     echo "Current city = $curr_city"
    endif

  endif
endif

# =================
# Prompt with pashua

  if ("$platypus_flag" == "1") then

    if ($single == 1) then
	set extra_msg = "Nothing";
	if ("$author" != "") 	set extra_msg = "$author";
	if ("$city" != "") 	set extra_msg = "$extra_msg, $city";
    endif


    if ("$curr_author" == "") set curr_author = "$author" # If empty, use default, if found

      if (-e $pashua_saved_keywords) set saved_keywords = `cat $pashua_saved_keywords`
      cp $pashua_config_template $pashua_config
      cat $pashua_config | sed -e "s/newtitle.default.*/newtitle.default = $curr_title/" > /tmp/mmmtmp
      if ("$single" == "1") then
	    cat /tmp/mmmtmp | sed -e "s/oldkeywords.text.*/oldkeywords.text = $curr_keys/" > /tmp/mmmtmp2
      else # multiple files
	    cat /tmp/mmmtmp | sed -e "s/keywordheader.text.*/keywordheader.text = Multiple files chosen/" -e "/oldkeywords/d" -e "/savekeys/d" -e "s/#multiprompt/multiprompt/" > /tmp/mmmtmp2
      endif

      if ("$showimage" == "1" && "$single" == "1") then
	    cat /tmp/mmmtmp2 | sed -e "s/#thumbimage/thumbimage/" -e "s%thumbimage.path.*%thumbimage.path = $argv[1]%" > /tmp/mmmtmp
	    mv /tmp/mmmtmp /tmp/mmmtmp2
      endif

      cat /tmp/mmmtmp2 | sed -e "s/newauthor.default.*/newauthor.default = $curr_author/" > /tmp/mmmtmp

      if (-s $pashua_saved_keywords) then # add saved keywords, and remove default blank
	    cat /tmp/mmmtmp | sed "/newkeywords.option  =/d" > /tmp/mmmtmp2 # NB double space before =
	    cat /tmp/mmmtmp2 | sed "/NEWKEYSHERE/r $pashua_saved_keywords" > /tmp/mmmtmp
      endif

      if ("$individualprompt" == "1") then
	    cat /tmp/mmmtmp | sed -e "s/.*thumbimage.label.*/thumbimage.label = $argv[1]:t/" > /tmp/mmmtmp2
        cat /tmp/mmmtmp2 | sed -e "s/#abort/abort/" > /tmp/mmmtmp
      endif

      cp /tmp/mmmtmp $pashua_config

      $pashua "$pashua_config" > "$pashua_output"

  else # not platypus
    echo -n "Keywords to append : "
    set ret = "2$<"
    #echo "RET $ret"
  endif
  ##echo $ret > jjj.jjj 

	set cancel = `cat $pashua_output | grep "cancel=" | sed -e s/cancel=//`
	if ("$cancel" == "1") goto exit
	set retcode = `cat $pashua_output | grep "enter=" | sed -e s/enter=//`
	set keywords = `cat $pashua_output | grep "newkeywords=" | sed -e s/newkeywords=//`
	set title = `cat $pashua_output | grep "newtitle=" | sed -e s/newtitle=//`
	set author = `cat $pashua_output | grep "newauthor=" | sed -e s/newauthor=//`
	set replace = `cat $pashua_output | grep "replace=" | sed -e s/replace=//`
	set savekeys = `cat $pashua_output | grep "savekeys=" | sed -e s/savekeys=//`
	set clearkeys = `cat $pashua_output | grep "clearkeys=" | sed -e s/clearkeys=//`
	set multiprompt = `cat $pashua_output | grep "multiprompt=" | sed -e s/multiprompt=//`
	set abort = `cat $pashua_output | grep "abort=" | sed -e s/abort=//`
	if ("$replace" == "Replace keywords") set append = " "
	if ("$abort" == "Abort") goto exit
		echo "Abort : $abort"


if ("$savekeys" == "Save updated keywords") then
  if ("$replace" == "Replace keywords") then
    echo "newkeywords.option =  $keywords" >> $pashua_saved_keywords # Append latest keywords + new to save set
  else
    echo "newkeywords.option =  $curr_keys,$keywords" >> $pashua_saved_keywords # Append latest keywords + new to save set
  endif
endif

if ("$savekeys" == "Save current keywords") echo "newkeywords.option =  $curr_keys" >> $pashua_saved_keywords # Append latest keywords to save set
if ("$savekeys" == "Clear saved keywords" && -e $pashua_saved_keywords) \rm $pashua_saved_keywords # Delete save set

# Second prompt box

  if ("$retcode" == "2") then # Append and get title (second prompt box)
    if ("$platypus_flag" == "1") then
      set prompt_string = "Enter title for image : "
      set def_text = "$curr_title";
      if ("$single" != "1") set def_text = "Multiple files chosen, title will be set in all";

      set ret = `"$ccd" inputbox --title "$prog" --informative-text "$prompt_string" --text "$def_text" --button3 "Cancel" --button2 "Replace keywords and title" --button1 "Append keywords, replace title" --float`
    else
      echo -n "Title : "
      set ret = "1$<"
    endif
    set retcode = `echo $ret | cut -c1`
    if ("$retcode" == "3") then # Cancel
      goto exit
    endif

    set title = `echo $ret | cut -c2-`
    if (`echo $title | grep -i "Multiple files chosen"` != "") set title = "";

    if ("$retcode" == "2") then # Replace
      set append = ""
    endif

  endif

# =================================
# Prompt for every file of a bunch

if ("$multiprompt" == "1") then

        set filecount = 0
	    if (-e $filelist) rm $filelist
        set iopt=0

        argloop:
        @ iopt ++
        set arg = "$argv[$iopt]"
        if (-d "$arg") then
          foreach file ("$arg"/*)
	        if (-d "$file") goto skiploop
	        echo "$file" >> $filelist
            @ filecount ++
          end
        else
	      echo "$arg" >> $filelist
          @ filecount ++
        endif
        if ($iopt < $#argv) goto argloop
        skiploop:
        ##echo "File count : $filecount $filelist";

    ###else 
        ###set filecount = 1;
    ###endif

    # Stored in file because I can't make append to an array !

    set total = `cat $filelist | wc -l`

    set icount = 0

loop:
        @ icount ++
        set nextfile = `cat $filelist | sed -n "$icount p"`

	echo nnn $nextfile

        "$macmetamod" multiprompt "$nextfile"

    if ($icount < $total) goto loop

endif # end multiprompt

if ("$multiprompt" == "1") exit;

# =============================================================== Progress meter
doit:

#set echo


    if ("$single" != "1") then # Count files

	set filecount = 0
	set iopt=0
	argloop3:
	@ iopt ++
	set arg = "$argv[$iopt]"
	if (-d "$arg") then
	  foreach file ("$arg"/*)
	    @ filecount ++
      end

	else
	  @ filecount ++
	endif
    if ($iopt < $#argv) goto argloop3
    
    #echo "File count : $filecount";

    else 
	  set filecount = 1;
    endif
   
# ===============================================================
# Modify each file in turn

set filelist = ""
set iopt=0
set workDir = "$argv[1]:h";
argloop2:
  @ iopt ++
  set arg = "$argv[$iopt]"
  set listlen = `echo $filelist $arg:t | wc -c`
  if ($listlen > $maxWordLength) then
    $pashua "$pashua_toomany"
    goto exit
  endif
  set filelist = "$filelist "\""$arg:t"\"

if ($iopt < $#argv) goto argloop2
skip2:
# Build up exifmod string

set exifmod = "$exifmod $overwrite_string"
if ("$author" == "" && "$single" == "1") set author = " ";
if ("$author" != "") set exifmod = "$exifmod -author "\""$author"\"
if ("$keywords" == "") set keywords = " ";
if ("$keywords" != "") set exifmod = "$exifmod -keywords "\""$keywords"\""$append"
if ("$title" == "" && "$single" == "1") set title = " ";
if ("$title" != "") set exifmod = "$exifmod -title "\""$title"\"
if ("$location" == "") set location = " ";
if ("$location" != "") set exifmod = "$exifmod -location "\""$location"\"
if ("$city" == "") set city = " ";
if ("$city" != "") set exifmod = "$exifmod -city "\""$city"\"

if ("$meter" == "0") then # Don't show meter (ie single file)

echo "single"

    ##cd "$workDir"
    eval $exifmod $filelist

else # Multiple files

echo "multiple $workDir"

	 cd "$workDir"
     eval $exifmod -q -v0 $filelist

endif

# =========================================

exit:

# Tidy up (Kill lots of procs in case .cshrc started something)

if ($bubble_flag != 0) then
   kill -9 $bubpid
endif

exit
